use Test::More tests => 1;

BEGIN {
use_ok( 'Perl::Critic::Policy::Variables::RequireHungarianNotation' );
}

diag( "Testing Perl::Critic::Policy::Variables::RequireHungarianNotation $Perl::Critic::Policy::Variables::RequireHungarianNotation::VERSION" );
